<template>
  <el-bus-crud ref="crud" v-bind="tableConfig" />
</template>

<script>
import cloneDeep from 'lodash.clonedeep'
import { getSortConfig } from '@/utils/form-item-config'
export default {
  data() {
    return {
      originalList: [],
      expandIds: [],
      tableConfig: {
        url: 'flower/api/flower/category/tree',
        hasPagination: false,
        saveQuery: false,
        isTree: true,
        hasView: false,
        canNewChild: (row) => !row.parentId,
        dialogAttrs: {
          width: '70%',
        },
        /**
         * 优化树形数据量较大渲染卡顿的问题
         * 使用tree的懒加载后更新数据刷新树比较麻烦，暂时不考虑这个方案
         * 由于该分类列表最多两级
         * 目前在渲染树时先将每个父节点下的子元素children取第一个渲染树结构，然后在展开节点时通过id找到原始的children做替换
         * 考虑到更新数据时某些节点可能已经展开，已经展开的节点不能取第一个子元素，所以要记录展开的节点
         * 1.在expandChange事件中更新展开的节点列表
         * 2.当通过搜索条件搜索时，如果某些父节点不在搜索结果中，也要将它们从展开的节点中去除
         */
        afterRequest: (list) => {
          this.originalList = cloneDeep(list)
          this.expandIds = this.expandIds.filter((i) =>
            list.find((item) => item.id === i)
          )
          return list.map((i) => ({
            ...i,
            childrenCount: Array.isArray(i.children) ? i.children.length : 0,
            children: this.expandIds.includes(i.id)
              ? i.children
              : i.children.slice(0, 1),
          }))
        },
        tableEventHandlers: {
          expandChange: (row, expand) => {
            if (expand) {
              if (!this.expandIds.includes(row.id)) {
                this.expandIds.push(row.id)
              }
              row.children =
                this.originalList.find((i) => i.id === row.id)?.children || []
            } else {
              const index = this.expandIds.indexOf(row.id)
              if (index !== -1) {
                this.expandIds.splice(index, 1)
              }
            }
          },
        },
        beforeOpen(row, isNew) {
          if (isNew && row.name) {
            row.parentName = row.name
          }
          if (!isNew && !row.parentId && row.parentName) {
            row.parentName = ''
          }
        },
        extraParentKeys: ['parentName', 'levelLimit'],
        tableAttrs: {
          rowKey: 'id',
        },
        columns: [
          { label: '分类名称', prop: 'name' },
          {
            label: '分类图片',
            formatter: (row) => (
              <el-bus-image
                style="width:50px;height:50px"
                lazy={true}
                src={row.imageUrl}
              />
            ),
          },
          { label: '排序', prop: 'sortBy' },
          { label: '品种数量', formatter: (row) => row.childrenCount },
          {
            label: '是否显示',
            formatter: (row) => (
              <el-switch
                value={row.shown}
                onChange={this.onShownChange.bind(this, row)}
              ></el-switch>
            ),
          },
        ],
        searchForm: [
          {
            type: 'row',
            items: [{ label: '分类名称', id: 'name', type: 'input' }],
          },
        ],
        form: [
          {
            label: '上级分类：',
            id: 'parentName',
            type: 'input',
            readonly: true,
            hidden: (row) => !row.parentName,
          },
          {
            label: '分类名称：',
            id: 'name',
            type: 'input',
            rules: {
              required: true,
              message: '请输入分类名称',
              trigger: 'blur',
            },
          },
          {
            label: '分类图片：',
            id: 'imageUrl',
            type: 'bus-upload',
            el: {
              listType: 'picture-card',
              limit: 1,
              limitSize: 2,
              tipText: '建议上传164*164的图片，大小不超过2M',
              valueType: 'string',
            },
            rules: { required: true, message: '请上传分类图片' },
          },
          {
            label: '等级：',
            id: 'levelLimit',
            type: 'bus-select-dict',
            el: {
              code: 'FLOWER_LEVEL',
              multiple: true,
              style: 'width:100%',
            },
            rules: { required: true, message: '请选择等级' },
            inputFormat: (row) => {
              if ('levelLimit' in row) {
                return row.levelLimit ? row.levelLimit.split(',') : []
              }
            },
            outputFormat: (val) => {
              return Array.isArray(val) ? val.join(',') : null
            },
            hidden: (row) => row.parentName,
          },
          {
            label: '分类颜色：',
            id: 'color',
            type: 'input',
            hidden: (row) => !row.parentName,
          },
          {
            label: '规格：',
            id: 'unit',
            type: 'input',
            hidden: (row) => !row.parentName,
          },
          {
            type: 'row',
            items: [
              {
                label: 'A级重量：',
                id: 'weightA',
                type: 'input-number',
                el: {
                  min: 0,
                  precision: 1,
                  controls: false,
                },
                unit: 'kg',
                hidden: (row) =>
                  !row.parentName ||
                  !row.levelLimit ||
                  !row.levelLimit.includes('A'),
                rules: {
                  required: true,
                  message: '请输入A级重量',
                  trigger: 'blur',
                },
              },
              {
                label: 'B级重量：',
                id: 'weightB',
                type: 'input-number',
                el: {
                  min: 0,
                  precision: 1,
                  controls: false,
                },
                unit: 'kg',
                hidden: (row) =>
                  !row.parentName ||
                  !row.levelLimit ||
                  !row.levelLimit.includes('B'),
                rules: {
                  required: true,
                  message: '请输入B级重量',
                  trigger: 'blur',
                },
              },
              {
                label: 'C级重量：',
                id: 'weightC',
                type: 'input-number',
                el: {
                  min: 0,
                  precision: 1,
                  controls: false,
                },
                unit: 'kg',
                hidden: (row) =>
                  !row.parentName ||
                  !row.levelLimit ||
                  !row.levelLimit.includes('C'),
                rules: {
                  required: true,
                  message: '请输入C级重量',
                  trigger: 'blur',
                },
              },
              {
                label: 'D级重量：',
                id: 'weightD',
                type: 'input-number',
                el: {
                  min: 0,
                  precision: 1,
                  controls: false,
                },
                unit: 'kg',
                hidden: (row) =>
                  !row.parentName ||
                  !row.levelLimit ||
                  !row.levelLimit.includes('D'),
                rules: {
                  required: true,
                  message: '请输入D级重量',
                  trigger: 'blur',
                },
              },
              {
                label: 'E级重量：',
                id: 'weightE',
                type: 'input-number',
                el: {
                  min: 0,
                  precision: 1,
                  controls: false,
                },
                unit: 'kg',
                hidden: (row) =>
                  !row.parentName ||
                  !row.levelLimit ||
                  !row.levelLimit.includes('E'),
                rules: {
                  required: true,
                  message: '请输入E级重量',
                  trigger: 'blur',
                },
              },
              {
                label: 'O级重量：',
                id: 'weightO',
                type: 'input-number',
                el: {
                  min: 0,
                  precision: 1,
                  controls: false,
                },
                unit: 'kg',
                hidden: (row) =>
                  !row.parentName ||
                  !row.levelLimit ||
                  !row.levelLimit.includes('O'),
                rules: {
                  required: true,
                  message: '请输入O级重量',
                  trigger: 'blur',
                },
              },
            ],
          },
          {
            ...getSortConfig(),
          },
        ],
      },
    }
  },
  head() {
    return {
      title: '分类管理',
    }
  },
  methods: {
    onShownChange(row, e) {
      const url = e
        ? 'flower/api/flower/category/tree/shown'
        : 'flower/api/flower/category/tree/hidden'
      const text = e ? '显示' : '隐藏'
      this.$elBusUtil
        .confirm(`确定要${text}这个分类吗？`)
        .then(async () => {
          const { code } = await this.$elBusHttp.request(url, {
            params: {
              id: row.id,
            },
          })
          if (code === 0) {
            this.$message.success(`${text}成功`)
            this.$refs.crud.getList()
          }
        })
        .catch(() => {})
    },
  },
}
</script>

<style lang="scss" scoped>
.category-list {
}
</style>
